@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2006-2016 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 2006-2019 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -42,17 +42,6 @@
   SaverRunner *_parent;
   NSString *_saverName;
   XScreenSaverView *_saverView;
-
-  /* When a the SaverViewController is presented, iOS automatically changes
-     the status bar orientation. (And, by extension, the notification center
-     orientation.) But there's no willPresentAsModal: event for a
-     UIViewController so that it knows when this is going to happen, and the
-     other event handlers occur after the status bar is changed. So save the
-     orientation just before the view controller is modal-presented, and
-     restore the proper status bar orientation just before the saverView is
-     created so it can pick it up in didRotate:. */
-  // UIInterfaceOrientation _storedOrientation;
-
   BOOL _showAboutBox;
   UIView *aboutBox;
   NSTimer *splashTimer;
