@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2006-2016 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 2006-2019 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -41,14 +41,19 @@ extern void check_gl_error (const char *type);
 @implementation XScreenSaverGLView
 
 
-#ifdef USE_IPHONE
 /* With GL programs, drawing at full resolution isn't a problem.
  */
-- (CGFloat) hackedContentScaleFactor
+- (CGFloat) hackedContentScaleFactor:(BOOL)fonts_p
 {
+# ifdef USE_IPHONE
   return [self contentScaleFactor];
+# else
+  return self.window.backingScaleFactor;
+# endif
 }
 
+# ifdef USE_IPHONE
+
 - (BOOL)ignoreRotation
 {
   return FALSE;		// Allow xwindow and the glViewport to change shape
@@ -346,8 +351,8 @@ init_GL (ModeInfo *mi)
 
   // Caller expects a pointer to an opaque struct...  which it dereferences.
   // Don't ask me, it's historical...
-  static int blort = -1;
-  return (void *) &blort;
+  static GLXContext blort = (GLXContext) -1;
+  return &blort;
 }
 
 
