@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2006-2019 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 2006-2020 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -18,6 +18,9 @@
 #import <QuartzCore/QuartzCore.h>
 #import <sys/mman.h>
 #import <zlib.h>
+#ifdef LOG_STACK
+# include <execinfo.h>
+#endif
 #import "XScreenSaverView.h"
 #import "XScreenSaverConfigSheet.h"
 #import "Updater.h"
@@ -51,6 +54,10 @@
 # define DO_GC_HACKERY
 #endif
 
+#undef countof
+#define countof(x) (sizeof((x))/sizeof((*x)))
+
+
 /* Duplicated in xlockmoreI.h and XScreenSaverGLView.m. */
 extern void clear_gl_error (void);
 extern void check_gl_error (const char *type);
@@ -377,6 +384,54 @@ add_default_options (const XrmOptionDescRec *opts,
 }
 
 
+static void sighandler (int sig)
+{
+  const char *s = strsignal(sig);
+  if (!s) s = "Unknowng";
+# ifdef USE_IPHONE
+  jwxyz_abort ("Signal: %s", s);	// Throw NSException, show dialog
+# else
+  NSLog (@"Signal: %s", s);		// Just make sure it is logged
+
+  // Log stack trace too.
+  // Same info shows up in Library/Logs/DiagnosticReports/ScreenSaverEngine*
+#  ifdef LOG_STACK
+  void *stack [20];
+  int frames = backtrace (stack, countof(stack));
+  char **strs = backtrace_symbols (stack, frames);
+  NSMutableArray *backtrace = [NSMutableArray arrayWithCapacity:frames];
+  for (int i = 2; i < frames; i++) {
+    if (strs[i])
+      [backtrace addObject:[NSString stringWithUTF8String: strs[i]]];
+  }
+  // Can't embed newlines in the message for /usr/bin/log
+  NSLog(@"Stack:\\n\t%@", [backtrace componentsJoinedByString:@"\\n\t"]);
+  // free (strs);
+#  endif // LOG_STACK
+
+  signal (sig, SIG_DFL);
+  kill (getpid (), sig);
+# endif
+}
+
+static void catch_signals (void)
+{
+  signal (SIGINT,  sighandler);
+  signal (SIGQUIT, sighandler);
+  signal (SIGILL,  sighandler);
+  signal (SIGTRAP, sighandler);
+  signal (SIGABRT, sighandler);
+  signal (SIGEMT,  sighandler);
+  signal (SIGFPE,  sighandler);
+  signal (SIGBUS,  sighandler);
+  signal (SIGSEGV, sighandler);
+  signal (SIGSYS,  sighandler);
+  signal (SIGTERM, sighandler);
+  signal (SIGXCPU, sighandler);
+  signal (SIGXFSZ, sighandler);
+}
+
+
 - (id) initWithFrame:(NSRect)frame
            saverName:(NSString *)saverName
            isPreview:(BOOL)isPreview
@@ -384,6 +439,7 @@ add_default_options (const XrmOptionDescRec *opts,
   if (! (self = [super initWithFrame:frame isPreview:isPreview]))
     return 0;
   
+  catch_signals();
   xsft = [self findFunctionTable: saverName];
   if (! xsft) {
     [self release];
@@ -444,6 +500,27 @@ add_default_options (const XrmOptionDescRec *opts,
 }
 
 
+#ifndef USE_IPHONE
+/* On 10.15, if "use random screen saver" is checked, then startAnimation
+   is never called.  This may be related to Apple's buggy code in 
+   ScreenSaverEngine calling nonexistent beginExtensionRequestWithUserInfo,
+   but on 10.15 we're not even running in that process: now we're in the
+   not-at-all-ominously-named legacyScreenSaver process.
+ */
+- (void) viewDidMoveToWindow
+{
+  if (self.window)
+    [self startAnimation];
+}
+
+- (void) viewWillMoveToWindow:(NSWindow *)window
+{
+  if (window == nil)
+    [self stopAnimation];
+}
+#endif  // USE_IPHONE
+
+
 #ifdef USE_TOUCHBAR
 - (id) initWithFrame:(NSRect)frame
            saverName:(NSString *)saverName
@@ -581,6 +658,8 @@ add_default_options (const XrmOptionDescRec *opts,
 
 - (void) startAnimation
 {
+  if ([self isAnimating]) return;  // macOS 10.15 stupidity
+
   NSAssert(![self isAnimating], @"already animating");
   NSAssert(!initted_p && !xdata, @"already initialized");
 
@@ -940,9 +1019,17 @@ screenhack_do_fps (Display *dpy, Window w, fps_state *fpst, void *closure)
   CGFloat s = self.window.backingScaleFactor;
 # endif
 
+  /* This notion of "scale fonts differently than the viewport seemed
+     like it made sense for BSOD but it makes -fps text be stupidly
+     large for all other hacks. So instead let's just make BSOD not
+     be lowrez. There are no other lowrez hacks that make heavy use
+     of fonts. */
+  fonts_p = 0;
+
   if (_lowrez_p && !fonts_p) {
     NSSize b = [self bounds].size;  // This is in points, not pixels
     CGFloat wh = b.width > b.height ? b.width : b.height;
+    wh *= s;  // points -> pixels
 
     // Scale down to as close to 1024 as we can get without going under,
     // while keeping an integral scale factor so that we don't get banding
@@ -1644,14 +1731,15 @@ gl_check_ver (const struct gl_version *caps,
       if (_lowrez_p) {
         resized_p = YES;
 
-# if !defined __OPTIMIZE__ || TARGET_IPHONE_SIMULATOR
         NSSize  b = [self bounds].size;
         CGFloat s = self.hackedContentScaleFactor;
-#  ifdef USE_IPHONE
+# ifdef USE_IPHONE
         CGFloat o = self.contentScaleFactor;
-#  else
+# else
         CGFloat o = self.window.backingScaleFactor;
-#  endif
+# endif
+
+# if !defined __OPTIMIZE__ || TARGET_IPHONE_SIMULATOR
         if (o != s)
           NSLog(@"lowrez: scaling %.0fx%.0f -> %.0fx%.0f (%.02f)",
                 b.width * o, b.height * o,
@@ -1866,6 +1954,18 @@ gl_check_ver (const struct gl_version *caps,
     [self handleException: e];
   }
 # endif // USE_IPHONE
+
+# if 0
+  {
+    static int frame = 0;
+    if (++frame == 100) {
+      fprintf(stderr,"BOOM\n");
+      int y = 0;
+      //    int aa = *((int*)y);
+      int x = 30/y;
+    }
+  }
+# endif
 }
 
 
