@@ -101,14 +101,17 @@ static const char *words[] =
   "abstraction",
   "acid",
   "addiction",
+  "affluenza",
   "alertness",
   "Algeria",
+  "antifa",
   "anxiety",
   "aorta",
   "argyle socks",
   "attrition",
   "axis of evil",
   "bamboo",
+  "banana slug",
   "bangle",
   "bankruptcy",
   "baptism",
@@ -122,6 +125,7 @@ static const char *words[] =
   "boobs",
   "booty",
   "bread",
+  "brogrammers",
   "bubba",
   "burrito",
   "California",
@@ -131,8 +135,10 @@ static const char *words[] =
   "carnage",
   "children",
   "chocolate",
+  "chupacabra",
   "CLONE",
   "cock",
+  "congress",
   "constriction",
   "contrition",
   "cop",
@@ -153,10 +159,10 @@ static const char *words[] =
   "despair",
   "desperation",
   "disease",
-  "disease",
+  "DNA Lounge",
   "doberman",
   "DOOM",
-  "dreams",
+  "dot com",
   "dreams",
   "drugs",
   "easy",
@@ -175,9 +181,10 @@ static const char *words[] =
   "flatulence",
   "fluff",
   "fnord",
+  "followers",
+  "frak",
   "freedom",
   "fruit",
-  "fruit",
   "futility",
   "gerbils",
   "GOD",
@@ -198,8 +205,12 @@ static const char *words[] =
   "icepick",
   "identity",
   "ignorance",
+  "illuminati",
   "importance",
   "individuality",
+  "influence",
+  "influencers",
+  "influenza",
   "inkling",
   "insurrection",
   "intoxicant",
@@ -215,6 +226,7 @@ static const char *words[] =
   "lattice",
   "lawyer",
   "lemming",
+  "likes",
   "liquidation",
   "lobbyist",
   "love",
@@ -224,9 +236,11 @@ static const char *words[] =
   "malfunction",
   "marmot",
   "marshmallow",
+  "measles",
   "merit",
   "merkin",
   "mescaline",
+  "methane",
   "milk",
   "mischief",
   "mistrust",
@@ -281,6 +295,7 @@ static const char *words[] =
   "respect",
   "revolution",
   "roadrunner",
+  "rootkit",
   "rule",
   "savor",
   "scab",
@@ -290,6 +305,7 @@ static const char *words[] =
   "security",
   "sediment",
   "self worth",
+  "shadow profile",
   "sickness",
   "silicone",
   "slack",
@@ -316,17 +332,21 @@ static const char *words[] =
   "terrorism",
   "terrorist",
   "the impossible",
+  "the panopticon",
   "the unknown",
   "toast",
   "topography",
   "truism",
+  "truthiness",
   "turgid",
+  "twits",
   "underbrush",
   "underling",
   "unguent",
   "unusual",
   "uplink",
   "urge",
+  "vaccines",
   "valor",
   "variance",
   "vaudeville",
@@ -334,6 +354,7 @@ static const char *words[] =
   "vegetarian",
   "venom",
   "verifiability",
+  "very fine people",
   "viagra",
   "vibrator",
   "victim",
@@ -351,7 +372,6 @@ static const char *words[] =
   "words",
   "worm",
   "worship",
-  "worship",
   "Xanax",
   "Xerxes",
   "Xhosa",
