@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1999 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1999-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -833,7 +833,11 @@ compass_init (Display *dpy, Window window)
       st->xgwa.height > st->xgwa.width * 5)
     st->size2 = MAX(st->xgwa.width, st->xgwa.height);
 
-  if (st->size2 > 600) st->size2 = 600;
+  {
+    int max = 600;
+    if (st->xgwa.width > 2560) max *= 2;  /* Retina displays */
+    if (st->size2 > max) st->size2 = max;
+  }
 
   st->size = (st->size2 / 2) * 0.8;
 
@@ -953,6 +957,18 @@ compass_event (Display *dpy, Window window, void *closure, XEvent *event)
 static void
 compass_free (Display *dpy, Window window, void *closure)
 {
+  struct state *st = (struct state *) closure;
+  int i;
+  XFreeGC (dpy, st->ptr_gc);
+  XFreeGC (dpy, st->erase_gc);
+  for (i = 0; i < 4; i++)
+    if (st->discs[i]) {
+      XFreeGC (dpy, st->discs[i]->gc);
+      free (st->discs[i]);
+    }
+  if (st->ba) XFreePixmap (dpy, st->ba);
+  if (st->bb) XFreePixmap (dpy, st->bb);
+  free (st);
 }
 
 
